@@ -32,7 +32,7 @@
 #
 # To stop and clear all running sessions use:
 #
-#   perl qemu-web-desktop.pl --dir_snapshots=/tmp  --session_purge=1
+#   perl qemu-web-desktop.pl --dir_snapshots=/tmp  --service_purge=1
 #
 # To monitor all running sessions, use:
 #
@@ -70,13 +70,13 @@ use strict;
 use warnings qw( all );
 
 use CGI;                # use CGI.pm
+use Cwd 'abs_path';     # for abs_path
 use File::Temp      qw/ tempdir tempfile /;
 use File::Path      qw/ rmtree  /;
 use File::Basename  qw/ fileparse dirname /;
 use List::MoreUtils qw(uniq); # liblist-moreutils-perl
 use List::Util;
 use Sys::CPU;           # libsys-cpu-perl           for CPU::cpu_count
-use Sys::CpuLoad;       # libsys-cpuload-perl       for CpuLoad::load
 use JSON;               # libjson-perl              for JSON
 use IO::Socket::INET;
 use IO::Socket::IP;
@@ -94,6 +94,9 @@ use LWP::UserAgent;     # libwww-perl               for new and get(URL)
 use Net::Ping;          # perl                      to test server up-down
 use URI;                # liburi-perl               to split URL into parts
 
+#   Sys::CpuLoad          libsys-cpuload-perl       for CpuLoad::load
+#   NOTE: we trap import errors (not present in Arch)
+BEGIN { eval { require Sys::CpuLoad; }; Sys::CpuLoad->import; } # silently ignore when missing module
 
 # see http://honglus.blogspot.com/2010/08/resolving-perl-cgi-buffering-issue.html
 $| = 1;
@@ -119,9 +122,9 @@ use constant IS_CGI      => IS_MOD_PERL || exists $ENV{'GATEWAY_INTERFACE'};
 # we use a Hash to store the configuration. This is simpler to pass to functions.
 our %config;
 
-$config{version}                  = "25.01.25";  # year.month.day
+$config{version}                  = "25.02.28";  # year.month.day
 
-my $dirname = dirname(__FILE__);
+my $dirname = abs_path(dirname(__FILE__));
 
 # use local config when found
 if (-e "$dirname/../config.pl") {
@@ -154,11 +157,12 @@ if (-e "$dirname/../config.pl") {
 # each of these can display its configuration, and be stopped/cleaned.
 $config{service_monitor}          = 0;
 $config{session_stop}             = ""; # contains a json ref, stop service
-$config{session_purge}            = 0;  # when true stop/clean all
+$config{service_purge}            = 0;  # when true stop/clean all
 $config{oneshot}                  = 0;
 
 for(my $i = 0; $i < @ARGV; $i++) {
   $_ = $ARGV[$i];
+  if(! $_) { next; }
   if(/--help|-h$/) {
     print STDERR "$0: launch a QEMU/KVM machine in a browser window. Version $config{version}\n\n";
     print STDERR "Usage: $0 --option1=value1 ...\n\n";
@@ -166,7 +170,6 @@ for(my $i = 0; $i < @ARGV; $i++) {
     foreach my $key (sort keys %config) {
       print STDERR "  --$key=VALUE [$config{$key}]\n";
     }
-    print "\n(c) 2020- Emmanuel Farhi - GRADES - Synchrotron Soleil. AGPL3.\n";
     exit;
   } elsif (/--version|-v$/) {
     print STDERR "$0 $config{version}\n\n";
@@ -287,7 +290,7 @@ $config{server_name} = $session{server_name};
 
 # check input arguments values (not 'password')
 for ('machine','user','cpu','memory','gpu','snapshot_lifetime',
-  'session_stop','session_purge') {
+  'session_stop','service_purge') {
   my $val = $q->param($_);
   if (defined($val)) {
     if ( $val =~ /^([a-zA-Z0-9_.\-@\/: ]+)$/ ) {
@@ -300,7 +303,7 @@ for ('machine','user','cpu','memory','gpu','snapshot_lifetime',
 
 # these are the "input" to collect from the HTML FORM. Store in $session
 for ('machine','user','cpu','memory','gpu','config_script','snapshot_lifetime',
-  'session_stop','session_purge','password','oneshot','fallback_servers') {
+  'session_stop','service_purge','password','oneshot','fallback_servers') {
   my $val = $q->param($_);
   if (defined($val)) {
     $session{$_} = $val;
@@ -345,13 +348,19 @@ $output .= <<END_HTML;
   <img alt="DARTS" title="DARTS"
     src="$config{http}://$session{server_name}/$config{service}/images/darts_logo.png"
     align="right" height="128" width="173"></a>
-  <h1>Data Analysis Remote Treatment Service: $session{machine}</h1>
-  <hr><ul>
+  <h1>$config{layout_title}
 END_HTML
 
-# $output .= "<li>$ok Starting on $session{date}</li>\n";
-# $output .= "<li>$ok The server name is $session{server_name}.</li>\n";
-# $output .= "<li>$ok You are accessing this service from $session{remote_host}.</li>\n";
+if ($config{layout_level} >= 1) {
+  $output .= ": $session{machine}";
+}
+$output .= "</h1><hr><ul>";
+
+if ($config{layout_level} >= 2) {
+  $output .= "<li>$ok Starting on $session{date}</li>\n";
+  $output .= "<li>$ok The server name is $session{server_name}.</li>\n";
+  $output .= "<li>$ok You are accessing this service from $session{remote_host}.</li>\n";
+}
 
 # stop/purge sessions from FORM (not args) - authentication is not needed as it was checked via form
 if ($session{session_stop}) {
@@ -359,10 +368,11 @@ if ($session{session_stop}) {
   my $session_ref = session_load(\%config, $session{session_stop});
   if ($session_ref) {
     session_stop($session_ref);
+    my %session = %{ $session_ref };
+    print "Content-type:text/html\r\n\r\n";
+    print "$output\n\n";
+    print "<b>$ok Stopped $session{session_stop} $session{machine}</b> that was started on [$session{date}] by $session{user} $session{user_email}\n";
   }
-  print "Content-type:text/html\r\n\r\n";
-  print "$output\n\n";
-  print "OK Stopped $session{session_stop} $session{machine} that was started on [$session{date}] by $session{user} $session{user_email}\n";
   exit;
 }
 
@@ -372,17 +382,16 @@ if ($session{machine} =~ 'monitor') {
   $config{service_monitor} = 1;
 }
 
-if ($session{machine} =~ 'purge' or $session{session_purge}) {
-  $config{session_purge} = 1;
+if ($session{machine} =~ 'purge' or $session{service_purge}) {
+  $config{service_purge} = 1;
 }
 
 # handle 'admin'actions
-if (not $error and ($config{service_monitor} or $config{session_purge})) {
+if (not $error and ($config{service_monitor} or $config{service_purge})) {
   (my $out, my $err) = session_authenticate(\%config, \%session);
   
   if (not $err) { # must be identified
     my $user  = $session{user};
-    if ($session{session_purge}) { $user = $session{session_purge}; }
     my @admin = @{ $config{user_admin} };
     my $isadmin = 0;
     $session{machine} = "administration";
@@ -392,15 +401,10 @@ if (not $error and ($config{service_monitor} or $config{session_purge})) {
     
     if ($config{service_monitor}) {
       $output = service_monitor(\%config, $output, $isadmin, $user);
-    } elsif ($config{session_purge}) {
-      if (not $isadmin) {
-        $err .= "User $user is not among the 'user_admin' list";
-      } else {
-        $config{snapshot_lifetime} = 1;
-        print STDERR $0." [$session{date}] Stopping all session, requested by $user\n";
-        service_housekeeping(\%config);
-        $output .= '</ul><h1>OK: Purged all</h1>';
-      }
+    } elsif ($config{service_purge}) {
+      print STDERR $0." [$session{date}] Stopping all session\n";
+      service_purge(\%config);
+      $output .= '</ul><h1>OK Purged all</h1>';
     }
   } 
   
@@ -423,28 +427,31 @@ if (not $error and ($config{service_monitor} or $config{session_purge})) {
 # ------------------------------------------------------------------------------
 $session{memory} = $session{memory}*1024; # GB -> MB
 
-if ($config{service_monitor} or $config{session_purge}) {
+if ($config{service_monitor} or $config{service_purge}) {
   # admin actions must always be possible
 } else {
   if ($config{session_nb} > $config{service_max_session_nb}) {
     $error .= "Too many active sessions $config{session_nb}. Max $config{service_max_session_nb}. Try again later.";
     $need_fallback_server = 1;
   }
-  if (Sys::CPU::cpu_count()-Sys::CpuLoad::load() < $session{cpu}) {
-    $error .= "Not enough free CPU's. Try again later.\n";
-    $need_fallback_server = 1;
-  }
-  if (Sys::CpuLoad::load() / Sys::CPU::cpu_count() > $config{service_max_load}) {
-    $error .= "Server load exceeded. Try again later.\n";
-    $need_fallback_server = 1;
-  }
+  # ignore error when CpuLoad module is not installed (Arch)
+  eval {
+    if (Sys::CPU::cpu_count()-Sys::CpuLoad::load() < $session{cpu}) {
+      $error .= "Not enough free CPU's. Try again later.\n";
+      $need_fallback_server = 1;
+    }
+    if (Sys::CpuLoad::load() / Sys::CPU::cpu_count() > $config{service_max_load}) {
+      $error .= "Server load exceeded. Try again later.\n";
+      $need_fallback_server = 1;
+    }
+  };
   my ($total_memory_kb, $free_memory_kb, $avail_memory_kb) = get_freemem();
   if ($avail_memory_kb / 1024 < $session{memory}) { # in MB
-    $error .= "Not enough free memory. Try again later.\n";
+    $error .= "Not enough free memory ($avail_memory_kb). Try again later.\n";
     $need_fallback_server = 1;
   }
   if (not -e "$config{dir_machines}/$session{machine}") {
-    $error .= "Can not find virtual machine.\n";
+    $error .= "Can not find virtual machine $session{machine}.\n";
   }
   (my $nb_session, my $nb_cpu, my $nb_mem) = service_check_user_max(\%config, $session{user});
   if ($nb_session > $config{service_max_session_nb_per_user}) {
@@ -471,7 +478,9 @@ if (defined($session{gpu}) and $session{gpu} =~ /yes|gpu|true|1/i) {
     }
   }
   if ($session{gpu}) {
-    $output .= "<li>$ok Assigned GPU at PCI $session{gpu}.</li>\n";
+    if ($config{layout_level} >= 2) {
+      $output .= "<li>$ok Assigned GPU at PCI $session{gpu}.</li>\n";
+    }
   } else {
     $error .= "Can not find a free GPU as requested. Try again without.\n";
     $need_fallback_server = 1;
@@ -554,7 +563,7 @@ if ($need_fallback_server and length($session{fallback_servers})) {
       print STDERR $response->as_string();
     }
   } # else proceed with error message: no localhost cpu/mem/gpu resource left
-}
+} # fallback server handling (load levelling)
 
 # ------------------------------------------------------------------------------
 # User credentials checks, find communication ports
@@ -567,13 +576,17 @@ if ($need_fallback_server and length($session{fallback_servers})) {
 }
 
 if (defined($session{oneshot}) and $session{oneshot} =~ /yes|oneshot|true|1/i) {
-  $output .= "<li>$ok Using single login session (<b>one-shot</b> login).</li>\n";
+  if ($config{layout_level} >= 2) {
+    $output .= "<li>$ok Using single login session (<b>one-shot</b> login).</li>\n";
+  }
   $session{oneshot} = "yes";
   $session{token_vnc}= "";
   # must use one websockify per session
   $config{service_port_multiple} = 1;
 } else {
-  $output .= "<li>$ok Using multiple login session (reconnect/share).</li>\n";
+  if ($config{layout_level} >= 2) {
+    $output .= "<li>$ok Using multiple login session (reconnect/share).</li>\n";
+  }
   $session{oneshot} = "";
 }
 
@@ -630,13 +643,17 @@ if (not $error) {
   if ($session{machine} =~ /\.iso$/i) { # machine ends with .ISO
     $cmd      = "qemu-img create -f qcow2 $session{snapshot} $session{disk}G";
     $res      = `$cmd`; # execute command
-    $output   .= "<li>$ok Will use ISO from ";
+    if ($config{layout_level} >= 2) {
+      $output   .= "<li>$ok Will use ISO from ";
+    }
   } else {
     # qemu-img now requires to specify master "backing" file format with '-F FMT'
     if ($config{snapshot_use_master} and -w "$config{dir_machines}/$session{machine}") {
       # option to NOT use snapshot. Work on master when writable.
       $session{snapshot} = "$config{dir_machines}/$session{machine}";
-      $output  .= "<li>$ok Using master ";
+      if ($config{layout_level} >= 2) {
+        $output  .= "<li>$ok Using master (changes will be kept) ";
+      }
     } else {
       my $img_info = `qemu-img info $config{dir_machines}/$session{machine}`;
       my @backing_format = $img_info =~ /file format:\s*([a-z0-9]+)/i;
@@ -644,11 +661,15 @@ if (not $error) {
       $cmd      = "qemu-img create -F $F -b $config{dir_machines}/$session{machine}"
                 . " -f qcow2 $session{snapshot}";
       $res      = `$cmd`; # execute command
-      $output  .= "<li>$ok Creating snapshot from ";
+      if ($config{layout_level} >= 2) {
+        $output  .= "<li>$ok Creating snapshot from ";
+      }
     }
   }
-  $output .= "<a href='$config{http}://$session{server_name}/$config{service}/machines/$session{machine}' target=_blank>"
-  . "$session{machine}</a> as session $session{name}</li>\n";
+  if ($config{layout_level} >= 2) {
+    $output .= "<a href='$config{http}://$session{server_name}/$config{service}/machines/$session{machine}' target=_blank>"
+    . "$session{machine}</a> as session $session{name}</li>\n";
+  }
   
   # check for existence of cloned VM
   sleep(1); # make sure the VM has been cloned
@@ -753,10 +774,12 @@ if (not $error) {
   #   system("at now <<< '$cmd'")
   # $proc_qemu = system("echo '$cmd' | at now") || "";
   $proc_qemu = Proc::Background->new($cmd);
-  if (not $proc_qemu) {
+  if (not $proc_qemu or not $proc_qemu->pid) {
     $error  .= "Could not start QEMU/KVM for $session{machine}.\n";
   } else {
-    # $output .= "<li>$ok Started QEMU/KVM for $session{machine} with VNC.</li>\n";
+    if ($config{layout_level} >= 2) {
+      $output .= "<li>$ok Started QEMU/KVM for $session{machine} with VNC.</li>\n";
+    }
     push @{ $session{pid} }, $proc_qemu->pid;
   }
   sleep(1);
@@ -814,9 +837,9 @@ if (not $error) {
 
   # $proc_novnc = system("echo '$cmd' | at now") || "";
   $proc_novnc = Proc::Background->new($cmd);
-  
+    
   if ($config{service_port_multiple}) {
-    if (not $proc_novnc) {
+    if (not $proc_novnc or not $proc_novnc->pid) {
       $error .= "Could not start websockify/noVNC.\n";
     } else {
       # need to clear the multiple websockify calls
@@ -852,68 +875,87 @@ if (not $error) {
   # save session info only when no error if found
   session_save(\%session);
 
-  # $output .= "<li>$ok No error, all is fine.</li>\n";
-  $output .= "<li>$ok Connect to your machine at <a href=$session{url} target=_blank>$session{url}</a>.</li>\n";
-  if ($session{token_vnc} and $config{service_port_multiple}) {
-    $output .= "<li><b>$ok Security token is: $session{token_vnc}</b></li>\n";
+  if ($config{layout_level} >= 2) {
+    $output .= "<li>$ok Connect to your machine at <a href=$session{url} target=_blank>$session{url}</a>.</li>\n";
+  }
+  if ($session{token_vnc} and $config{service_port_multiple} and $config{layout_level} >= 2) {
+    # this is also displayed as an H2 tag below
+    $output .= "<li><b>$ok Connection password (noVNC) is: $session{token_vnc}</b></li>\n";
   }
   if ($session{snapshot_lifetime}) {
     my $datestring = localtime(time()+$session{snapshot_lifetime});
-    $output .= "<li>$ok You can use your machine until <b>$datestring.</b></li>\n";
+    if ($config{layout_level} >= 1) {
+      $output .= "<li>$ok You can use your session until <b>$datestring.</b></li>\n";
+    }
   }
   $output .= "</ul><hr>\n";
-  my $output_token = "";
+
+  # prepare some snippets
+  my $output_token      = ""; # security code if any
+  my $output_share      = ""; # how to share
+  my $output_persistent = ""; # show reconnect info
+  my $output_icon       = ""; # QR code
+  
   if ($session{token_vnc} and $config{service_port_multiple}) {
-    $output_token = "<h2>Security token: $session{token_vnc}</h2>";
+    $output_token = "<h2>Connection password: $session{token_vnc}</h2>";
   }
-  my $output_share = "";
-  my $output_persistent = "";
   if (not $session{oneshot}) {
     $output_share = "<button onClick=\"SelfCopy(this.id)\" id=\"$session{url} $session{token_vnc}\" style=\"padding:0;border:0;background:none;outline:none\"><img alt=\"copy\" src=\"$config{http}://$session{server_name}/$config{service}/images/share-button.png\" title=\"Copy link to clipboard, share it with your colleagues.\" height=40></button>";
-    $output_persistent = "<li>You can close the browser and reconnect any time (within life-time) with the link above.</li>" .
-    "<li>Select the <b><span style=\"color:green\">[Manage sessions]</span></b> item in the service login page to list, reconnect or abort your sessions.</li>" .
-    "<li>You can collaborate in the same session with your colleagues. Just send them the link above.</li>" .
-    "<li>Please <b>shut-down the machine properly</b> (do not just logout or suspend).</li>";
+    
+    if ($config{layout_level} >= 1) {
+      $output_persistent = "NOTES: <ul>" .
+      "<li>You can close the browser and reconnect any time (within life-time) with the link above.</li>" .
+      "<li>Select the <b><span style=\"color:green\">[Manage sessions]</span></b> item in the service login page to list, reconnect or abort your sessions.</li>" .
+      "<li>You can collaborate in the same session with your colleagues. Just send them the link above.</li>";
+    }
+  } elsif ($config{layout_level} >= 1) {
+    $output_persistent = "NOTES: <ul>";
   }
   
   # Conditional import of Text::QRCode module when installed
-  my $icon = "
-        <img alt='$session{machine}' title='$session{machine}'
+  if ($config{layout_level} >= 1) {
+    $output_icon = "<img alt='$session{machine}' title='$session{machine}'
         src='$config{http}://$session{server_name}/$config{service}/images/logo-system.png'
         align='center' border='1' height='128'>";
   
-  my $qr_enabled = eval
-  {
-    require Text::QRCode;
-    Text::QRCode->import();
-    1;
-  };
+    my $qr_enabled = eval
+    {
+      require Text::QRCode;
+      Text::QRCode->import();
+      1;
+    };
+    
+    if ($qr_enabled) {
+      # https://metacpan.org/pod/Text::QRCode
+      my $arrayref = Text::QRCode->new()->plot("$session{url}");
+      $session{qrcode} = join "\n", map { join '', @$_ } @$arrayref; 
+    
+      # code from HTML::QRCode
+      my $w = "<td style=\"border:0;margin:0;padding:0;width:3px;height:3px;background-color: white;\">";
+      my $b = "<td style=\"border:0;margin:0;padding:0;width:3px;height:3px;background-color: black;\">";
+   
+      $output_icon
+          = '<table style="margin:0;padding:0;border-width:0;border-spacing:0;">';
+      $output_icon
+          .= '<tr style="border:0;margin:0;padding:0;">'
+          . join( '', map { $_ eq '*' ? $b : $w } @$_ ) . '</tr>'
+          for (@$arrayref);
+      $output_icon .= '</table>';
+    }
+  } # qr code
   
-  if ($qr_enabled) {
-    # https://metacpan.org/pod/Text::QRCode
-    my $arrayref = Text::QRCode->new()->plot("$session{url}");
-    $session{qrcode} = join "\n", map { join '', @$_ } @$arrayref; 
-  
-    # code from HTML::QRCode
-    my $w = "<td style=\"border:0;margin:0;padding:0;width:3px;height:3px;background-color: white;\">";
-    my $b = "<td style=\"border:0;margin:0;padding:0;width:3px;height:3px;background-color: black;\">";
- 
-    $icon
-        = '<table style="margin:0;padding:0;border-width:0;border-spacing:0;">';
-    $icon
-        .= '<tr style="border:0;margin:0;padding:0;">'
-        . join( '', map { $_ eq '*' ? $b : $w } @$_ ) . '</tr>'
-        for (@$arrayref);
-    $icon .= '</table>';
+  if ($config{layout_level} >= 1) {
+    $output_persistent .= "<li>This session has been created on request, and is NOT kept. ". 
+      "Your work <b>must be saved elsewhere</b> " .
+      "(e.g. mounted disk, ssh/sftp, Dropbox, OwnCloud...).</li></ul>";
   }
-
+  
+  # generate main text for the user
   $output .= <<END_HTML;
     <h1>Hello $session{user} !</h1>
     
-    <p>Your machine $config{service} $session{machine} has just started. 
-    
     <div style="text-align: center;">
-      <a href=$session{url} target=_blank>$icon</a>
+      <a href=$session{url} target=_blank>$output_icon</a>
       <br><br>
       
       <table style="width:100%;"><tr>
@@ -930,24 +972,15 @@ if (not $error) {
       </form></td>
       
       </tr></table>
-        $output_token
+      $output_token
     </div>
-
     
     <p>
-    NOTES: <ul>
     $output_persistent
-    <li>The virtual machine is created on request, and not kept. 
-      Your work <b>must be saved elsewhere</b> 
-      (e.g. mounted disk, ssh/sftp, Dropbox, OwnCloud...).</li>
-    </ul></p>
+    </p>
 
     <hr>
-    <small><a href="$config{http}://$session{server_name}/$config{service}/" target=_blank>Data Analysis Remote Treatment Service</a> (c) 2020- - <a href="http://www.synchrotron-soleil.fr" target="_top">Synchrotron Soleil</a> - Thanks for using our data analysis services !</small>
-    <a href="http://www.synchrotron-soleil.fr" target="_top">
-    <img alt="SOLEIL" title="SOLEIL"
-    src="$config{http}://$session{server_name}/$config{service}/images/logo_soleil.png"
-    align="right" border="0" height="48"></a>
+    <small><a href="$config{http}://$session{server_name}/$config{service}/" target=_blank>$config{layout_title}</a> - <a href="https://gitlab.com/soleil-data-treatment/remote-desktop">$config{service})</a></small>
     </body>
     
   <script>
@@ -1026,11 +1059,14 @@ if (not $error and $proc_novnc and $proc_qemu and $session{oneshot}) {
 # - service_housekeeping
 # - service_check_user_max
 # - service_monitor
+# - service_purge
 # - session_email
 # - session_authenticate
 # - session_check_smtp
 # - session_check_imap
 # - session_check_ldap
+# - session_check_local
+# - session_check_custom
 # - flatten
 # - proc_getchildren
 # - proc_running
@@ -1333,7 +1369,10 @@ sub service_monitor {
 
   # first display server ID and usage
   my $cpu_count       = Sys::CPU::cpu_count();
-  my $cpu_load        = Sys::CpuLoad::load();
+  my $cpu_load        = 0;
+  eval {
+    $cpu_load        = Sys::CpuLoad::load();
+  };
   my $load            = $cpu_load  / $cpu_count;
   my ($total_memory_kb, $free_memory_kb, $avail_memory_kb) = get_freemem(); # in kB
   my $avail_memory_GB = $avail_memory_kb  / 1024/1024; # in GB
@@ -1372,7 +1411,7 @@ sub service_monitor {
       $out .= "<br><hr><br><h1>Active sessions [$config{session_nb}] (administration)</h1><table  border='1'>\n";
       $out .= "<tr><th>Start Date</th><th>Name</th><th>Machine</th><th>User</th>";
       $out .= "<th>CPU </th><th>Memory (MB)</th><th>GPU</th><th>lifetime (h)</th><th>URL</th>";
-      $out .= "<th>PID's <form action=\"$config{http}://$config{server_name}/cgi-bin/qemu-web-desktop.pl\" method=post target=_blank><input type=hidden name=session_purge value=$user><input type=submit value=\"STOP ALL\"></input></form> </th></tr>\n";
+      $out .= "<th>PID's <form action=\"$config{http}://$config{server_name}/cgi-bin/qemu-web-desktop.pl\" method=post target=_blank><input type=hidden name=service_purge value=$user><input type=submit value=\"STOP ALL\"></input></form> </th></tr>\n";
     }
 
     foreach my $snapshot (glob("$dir/$service"."_*")) {
@@ -1437,7 +1476,33 @@ sub service_monitor {
   } # for 1..2
   
   return $out;
-}
+} # service_monitor
+
+# service_purge(\%config): kill all running sessions 
+sub service_purge {
+  my $config_ref  = shift;
+  
+  if (not $config_ref) { return; }
+  my %config = %{ $config_ref };
+  
+  my $dir     = $config{dir_snapshots};
+  my $cfg     = $config{dir_cfg};
+  my $service = $config{service};
+  
+  foreach my $snapshot (glob("$dir/$service"."_*")) {
+      if (-d $snapshot) { # is a snapshot directory
+        print STDERR $0." [$session{date}] service_purge: snapshot: $snapshot\n";
+        my $snaphot_name = fileparse($snapshot); # just the session name
+        if (-e "$cfg/$snaphot_name.json") {
+          print STDERR $0." [$session{date}] service_purge: json found\n";
+          my $session_ref = session_load(\%config, "$cfg/$snaphot_name.json");
+          session_stop($session_ref);
+        }
+      }
+  } # for
+  
+  
+} # service_purge
 
 # session_email(\%config, \%session, $output): send an email with URL
 sub session_email {
@@ -1597,7 +1662,9 @@ sub session_authenticate {
     if (index($authenticated, "SUCCESS") > -1) {
       # $out .= "<li>$ok You are authenticated: $authenticated</li>\n";
     } elsif (not $authenticated) {
-      $out .= "<li><b><font color=orange>[WARNING]</font></b> Service is running without user authentication.</li>\n";
+      if ($config{layout_level} >= 2) {
+        $out .= "<li><b><font color=orange>[WARNING]</font></b> Service is running without user authentication.</li>\n";
+      }
       # no authentication configured...
     } else {
       $err  .= "User $session{user} failed authentication. Check your username / password:  $authenticated"; 
